' ***************************************************************************************************
' SSMS Scripter
' A plugin for SQL Server 2005 Management Studio that allows you to script stored procedures,
' user-defined functions, and views in the SQL 2000 Enterprise Manager
' style: if exists, drop; create. 
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' ***************************************************************************************************

Imports System
Imports System.Collections
Imports System.Windows.Forms
Imports System.Collections.Generic
Imports Microsoft.SqlServer.Management.UI.VSIntegration
Imports Microsoft.SqlServer.Management.UI.VSIntegration.ObjectExplorer
Imports System.Runtime.InteropServices
Imports System.Text
Imports Extensibility

<ComVisible(True)> _
Public NotInheritable Class PlugIn
    Implements IDTExtensibility2

    Friend IsFinished As Boolean = False
    Friend IsComplete As Boolean = False
    Friend IsMenuRegisteredForSProcNode As Boolean = False
    Friend IsMenuRegisteredForSProcFolder As Boolean = False
    Friend IsMenuRegisteredForViewNode As Boolean = False
    Friend IsMenuRegisteredForViewFolder As Boolean = False
    Friend IsMenuRegisteredForUDFScalerNode As Boolean = False
    Friend IsMenuRegisteredForUDFTableNode As Boolean = False
    Friend IsMenuRegisteredForUDFFolder As Boolean = False
    Friend IsMenuRegisteredForDatabase As Boolean = False
    Friend ObjectExplorer As IObjectExplorerService
    Friend NodesDict As Dictionary(Of String, MenuNode) = New Dictionary(Of String, MenuNode)
    Friend IsInvoked As Boolean = False
    Friend SyncObj As Object = New Object()


    ' *******************************************************************************************
    ' Class:        OnSelectionChanged
    ' Author:       Mark Gilbert
    ' Date:         11/23/2006
    ' Project:      SSMS Scripter
    ' Description:  This handler is registered dynamically by the OnConnection routine.  Through
    '               the TableMenuNode constructor, this adds an item to the context menu for
    '               the SSMS object that will link back to the SSMSScriptor form.
    ' *******************************************************************************************
    Private Sub OnSelectionChanged(ByVal sender As Object, ByVal e As NodesChangedEventArgs)

        ' TODO: I found that selecting Stored Procedures (or any of the folders) from the treeview ...
        ' on the left, and then right-clicking on a specific item from the Summary tab brings up the scripter
        ' with * for the object name.  The menu items seem to be defaulting to what you select in the
        ' tree, not what you select on the Summary pane.  I'm not sure at this point how to resolve this.  
        Dim mnCurrent As MenuNode = MenuNode.GetInstanceOnSelectedNode(Me)
    End Sub

    ' *******************************************************************************************
    ' Class:        OnConnection
    ' Author:       Mark Gilbert
    ' Date:         11/23/2006
    ' Project:      SSMS Scripter
    ' Description:  This method fires when SSMS first starts up, but before connection to a
    '               specific database server has been made.
    ' *******************************************************************************************
    Public Sub OnConnection(ByVal Application As Object, ByVal ConnectMode As Extensibility.ext_ConnectMode, ByVal AddInInst As Object, ByRef custom As System.Array) Implements Extensibility.IDTExtensibility2.OnConnection
        ObjectExplorer = ServiceCache.GetObjectExplorer()
        Dim typeOfProvider As Type = GetType(IObjectExplorerEventProvider)
        Dim pvdrCurrent As IObjectExplorerEventProvider = CType(ObjectExplorer.GetService(typeOfProvider), IObjectExplorerEventProvider)
        AddHandler pvdrCurrent.SelectionChanged, New NodesChangedEventHandler(AddressOf OnSelectionChanged)
    End Sub


#Region " Unimplemented Methods "
    Public Sub OnDisconnection(ByVal RemoveMode As Extensibility.ext_DisconnectMode, ByRef custom As System.Array) Implements Extensibility.IDTExtensibility2.OnDisconnection

    End Sub
    Public Sub OnAddInsUpdate(ByRef custom As System.Array) Implements Extensibility.IDTExtensibility2.OnAddInsUpdate

    End Sub
    Public Sub OnStartupComplete(ByRef custom As System.Array) Implements Extensibility.IDTExtensibility2.OnStartupComplete

    End Sub

    ' *******************************************************************************************
    ' Class:        OnBeginShutdown
    ' Author:       Mark Gilbert
    ' Date:         11/23/2006
    ' Project:      SSMS Scripter
    ' Description:  This method fires when SSMS begins to shut down.
    ' *******************************************************************************************
    Public Sub OnBeginShutdown(ByRef custom As System.Array) Implements Extensibility.IDTExtensibility2.OnBeginShutdown

    End Sub
#End Region

End Class
