' ***************************************************************************************************
' SSMS Scripter
' A plugin for SQL Server 2005 Management Studio that allows you to script stored procedures,
' user-defined functions, and views in the SQL 2000 Enterprise Manager
' style: if exists, drop; create. 
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' ***************************************************************************************************

Imports System
Imports System.Collections
Imports System.Windows.Forms
Imports Microsoft.SqlServer.Management.UI.VSIntegration
Imports Microsoft.SqlServer.Management.UI.VSIntegration.ObjectExplorer
Imports System.Runtime.InteropServices
Imports System.Text
Imports System.Threading

<ComVisible(False)> _
Friend NotInheritable Class ScripterMenuItem
    Inherits ToolsMenuItemBase

    Private m_plgMain As PlugIn
    Private WithEvents m_frmScripter As SSMSScriptor
    Private m_frmSplashScreen As SplashScreen
    Private m_tiSplashScreen As Thread



    ' *******************************************************************************************
    ' Method:       New
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   plugin
    ' Description:  This constructor simply initiates the local m_plugin variable with the
    '               incoming parameter.
    ' *******************************************************************************************
    Friend Sub New(ByVal plgCurrent As PlugIn)
        Me.m_plgMain = plgCurrent
    End Sub


    ' *******************************************************************************************
    ' Method:       Invoke
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   {none}
    ' Description:  This opens the SSMSScriptor form for the node selected in SSMS.
    ' Revisions:
    '   12/9/2006   Mark Gilbert moved the form variables for the scripter form and the splash
    '                   screen from here to private class-level.
    '   12/22/2006  Mark Gilbert modified the code to pass the authentication credentials from
    '                   SSMS to the Scripter.  Since the Server Name is part of these credentials,
    '                   he also modified the call to the SSMSScripter constructor to not pass that
    '                   separately.
    ' *******************************************************************************************
    Protected Overrides Sub Invoke()
        Dim mnCurrent As MenuNode

        If (Me.m_plgMain.IsInvoked) Then
            MessageBox.Show("Can't begin new scripting process until previous one is finished")
            Return
        End If

        mnCurrent = MenuNode.GetInstanceOnSelectedNode(Me.m_plgMain)
        If (Not IsNothing(mnCurrent)) Then
            Me.m_plgMain.IsInvoked = True
            Try
                ' Open the splash screen first; it will be closed by the ScripterLoadedEventHandler method
                Me.m_tiSplashScreen = New Thread(New ThreadStart(AddressOf Me.OpenSplashScreen))
                Me.m_tiSplashScreen.Start()


                Me.m_frmScripter = New SSMSScriptor(mnCurrent.Connection, _
                                                    mnCurrent.DatabaseName, _
                                                    mnCurrent.NodeType, _
                                                    mnCurrent.ObjectName)
                Me.m_frmScripter.Show()

            Finally
                Me.m_plgMain.IsInvoked = False
                Me.CloseSplashScreen()

            End Try

        End If
    End Sub


    ' *******************************************************************************************
    ' Method:       Clone
    ' Author:       Mark Gilbert
    ' Date:         11/25/2006
    ' Parameters:   {none}
    ' Description:  This is (apparently) an invalid method for the scripter, and as a result
    '               merely throws an exception.
    ' *******************************************************************************************
    Public Overrides Function Clone() As Object
        Throw New Exception("The 'Clone' method is not valid for the ScripterMenuItem class.")
    End Function



    ' *******************************************************************************************
    ' Method:       ScripterLoadedEventHandler
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This closes the splash screen once the scripter form has completely loaded.
    ' *******************************************************************************************
    Public Sub ScripterLoadedEventHandler() Handles m_frmScripter.ScripterLoaded
        Me.CloseSplashScreen()
    End Sub


    ' *******************************************************************************************
    ' Method:       OpenSplashScreen
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This opens the splash screen.  It has to open it as a dialog box so that it
    '               doesn't disappear right away.
    ' *******************************************************************************************
    Private Sub OpenSplashScreen()
        Me.m_frmSplashScreen = New SplashScreen()
        Me.m_frmSplashScreen.ShowDialog()
    End Sub


    ' *******************************************************************************************
    ' Method:       CloseSplashScreen
    ' Author:       Mark Gilbert
    ' Date:         12/9/2006
    ' Parameters:   {none}
    ' Description:  This does the work of actually closing the thread that the splash screen
    '               was opened in.  Since the current thread opened a new one for the splash
    '               screen, trying to access the screen itself will cause a "cross-thread"
    '               exception, so we just abort the thread itself, and the screen goes down too.
    ' *******************************************************************************************
    Private Sub CloseSplashScreen()
        If (Me.m_tiSplashScreen.IsAlive) Then Me.m_tiSplashScreen.Abort()
    End Sub
End Class
